package meerkat.voting.gui.controllersFX;

import javafx.fxml.FXML;
import javafx.geometry.HPos;
import javafx.geometry.Pos;
import javafx.scene.control.Button;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.Separator;
import javafx.scene.control.ToggleGroup;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;
import javafx.scene.text.TextAlignment;
import meerkat.protobuf.Voting;
import meerkat.voting.VotingBooth;
import meerkat.voting.controller.callbacks.VoterCancelThrowable;
import meerkat.voting.gui.ui.UIUtils;
import meerkat.voting.gui.ui.uicommands.RaceVotingUICommand;

import java.util.ArrayList;
import java.util.List;

//import meerkat.voting.gui.ui.UIUtils;

/**
 * Created by Vladimir Eliezer Tokarev on 8/27/2016.
 * ChannelChoiceController handle the behavior of select channel section screen
 */
public class VotingController {

//    private List<Integer> chanelValue;
//    private int pointer;
//    private boolean lock;

    List<Voting.BallotAnswer> answers = new ArrayList<>();
    List<Voting.BallotQuestion> questions = new ArrayList<>();
    int index = 0;
    RaceVotingUICommand command = (RaceVotingUICommand) VistaNavigator.getCurrentCommand();

    @FXML ToggleGroup answerGroup;
    @FXML private Text currentPage;
    @FXML private Text totalPage;
    @FXML private ScrollPane questionPane;
    @FXML private Button nextButton;

    @FXML
    public void initialize() {
        questions = command.getQuestions();
        answerGroup = new ToggleGroup();
        totalPage.setText(""+(questions.size()+1));
        showQuestion();
    }


    @FXML
    private void nextQuestion() {
        if (questions.size()==0) {
            questions = command.getQuestions();
            answerGroup = new ToggleGroup();
        }
        if (answerGroup.getSelectedToggle()!=null) {
            String s = answerGroup.getSelectedToggle().getUserData().toString();
            answers.add(UIUtils.translateStringAnswerToProtoBufMessageAnswer(s));
            index++;
        }
        if (nextButton.getText().equals("Confirm")) {
            index++;
        }
        if (index < questions.size()) {
            showQuestion();
        } else {
            if (index == questions.size()) {
                showSummary();
            } else {
                //return answers to main threads
                nextPane();
            }
        }
    }

    private void showQuestion() {
            Voting.BallotQuestion ballotQuestion = questions.get(index);
            //question.setText("This is question n. "+index);
            currentPage.setText("Page "+(index+1));
            answerGroup.getToggles().clear();
            VBox vbox = VistaNavigator.uiThread.showQuestionInFX(ballotQuestion, answerGroup);
            nextButton.setText("Next");
            questionPane.setContent(vbox);
            questionPane.setFitToHeight(true);
            //questionPane.getChildren().clear();
//            questionPane.getChildren().addAll(vbox);
    }

    private void showSummary() {
//        Voting.BallotQuestion ballotQuestion = questions.get(index);
        currentPage.setText("Page "+(index+1));
        answerGroup.getToggles().clear();
        VBox vbox = new VBox();//VistaNavigator.uiThread.showQuestionInFX(ballotQuestion, answerGroup);
        vbox.setPrefWidth(questionPane.getPrefWidth()-30);
        /*Set title of summary*/
        Text title = new Text("Almost done!");
        title.setFont(Font.font("System", FontWeight.BOLD, 24));
        vbox.getChildren().add(title);
        /*Set subtitle of summary*/
        Text subtitle = new Text("Review your selections. If you are satisfied, press confirm.");
        subtitle.setFont(Font.font("System", FontWeight.BOLD, 18));
        subtitle.setStyle("-fx-margin: 0 0 70 0;");
        vbox.getChildren().add(subtitle);
        /*Create summery*/
        for (int i=0; i<questions.size(); i++) {
            String qst = UIUtils.bytesToString(questions.get(i).getQuestion().getData());
            int a_i = (int) answers.get(i).getAnswer(0);
            String ans = UIUtils.bytesToString(questions.get(i).getAnswer(a_i-1).getData());
            Text qst_txt = new Text((i+1)+". "+qst);
            Text ans_txt = new Text(ans);
            Separator sep = new Separator();
            sep.setMaxWidth(vbox.getPrefWidth());
            vbox.getChildren().add(sep);
            vbox.getChildren().add(qst_txt);
            vbox.getChildren().add(ans_txt);
        }
        vbox.setAlignment(Pos.CENTER);
        nextButton.setText("Confirm");
        questionPane.setContent(vbox);
        questionPane.setFitToHeight(true);

    }


    @FXML
    private void goBack() throws VoterCancelThrowable{
        if (index == 0) {
            doCancel();
        } else {
            --index;
            answers.remove(index);
            nextQuestion();
        }
    }

    @FXML
    private void doCancel() throws VoterCancelThrowable{
        // TODO: 6/14/2017 check flow and decide what to do...
        VoterCancelThrowable e = new VoterCancelThrowable();
        command.getCallback().onFailure(e);
        //see what is done in the original (basically need to call the callback of the command onFailure()
    }


    private void nextPane() {
        VistaNavigator.loadVista(VistaNavigator.LOADING);
        VistaNavigator.uiThread.answerVotingScreen(command, answers);
    }


}
